import 'dart:async';
import 'package:xml/xml.dart';

import 'package:canada_post/src/models/canada_post_api_manager/canada_post_api_manager.dart';
import 'package:canada_post/src/models/canada_post_call/canada_post_call.dart';
import 'package:canada_post/src/models/mailing_scenario/mailing_scenario.dart';

class PriceCaller extends CanadaPostCall {

  PriceCaller({
    CanadaPostApiManager manager
  }) : super(
    endpoint: "ship/price",
    manager: manager, 
    method: 'POST',
    headers: {
      "Content-Type": "application/vnd.cpc.ship.rate-v3+xml",
      "Accept": "application/vnd.cpc.ship.rate-v3+xml"
    });

  Future<Map> shoot(MailingScenario mailingScenario) async {
    if (mailingScenario.services == null) {
      switch (mailingScenario.destination.type) {
        case "country-code":
          mailingScenario.services = [
            Service(serviceCode: "INT.XP"),
            Service(serviceCode: "INT.IP.AIR"),
            Service(serviceCode: "INT.IP.SURF"),
            Service(serviceCode: "INT.PW.ENV"),
            Service(serviceCode: "INT.PW.PAK"),
            Service(serviceCode: "INT.PW.PARCEL"),
            Service(serviceCode: "INT.SP.AIR"),
            Service(serviceCode: "INT.SP.SURF"),
            Service(serviceCode: "INT.TP"),
          ];
          break;
        case "zip-code":
          mailingScenario.services = [
            Service(serviceCode: "USA.EP"),
            Service(serviceCode: "USA.PW.ENV"),
            Service(serviceCode: "USA.PW.PAK"),
            Service(serviceCode: "USA.PW.PARCEL"),
            Service(serviceCode: "USA.SP.AIR"),
            Service(serviceCode: "USA.TP"),
            Service(serviceCode: "USA.TP.LVM"),
            Service(serviceCode: "USA.XP"),
          ];
          break;
        case "postal-code":
          mailingScenario.services = [
            Service(serviceCode: "DOM.RP"),
            Service(serviceCode: "DOM.EP"),
            Service(serviceCode: "DOM.XP"),
            Service(serviceCode: "DOM.XP.CERT"),
            Service(serviceCode: "DOM.PC"),
            Service(serviceCode: "DOM.LIB"),
          ];
          break;
      }
    }
    XmlBuilder builder = XmlBuilder();
    buildMailingScenario(builder, mailingScenario);

    String xml = builder.build().toXmlString();
    Map json = await super.toJson(super.call(xml));

    return json;
  }
}