import 'package:meta/meta.dart';
import 'package:xml/xml.dart';

Destination automaticDestination(String location) {
  if (location.startsWith(RegExp(r'[0-9]{1}'))) {
    return UnitedStatesDestination(location: location);
  }
  else if (location.length == 2 || location.length == 3) {
    return InternationalDestination(location: location);
  }
  else return DomesticDestination(location: location);
}

abstract class Destination {
  final String type;
  String kind;
  String location;

  Destination({@required this.type});
}

class InternationalDestination extends Destination {
  /// country-code, e.g.: "FR", "UK"
  final String location;

  final String kind = "country-code";

  InternationalDestination({
    @required this.location,
  }) : super(type: "international");
}

class UnitedStatesDestination extends Destination {
  /// zip-code, e.g.: "90210"
  final String location;
  
  final String kind = "zip-code";
  
  UnitedStatesDestination({
    @required this.location,
  }) : super(type: "united-states");
}

class DomesticDestination extends Destination {
  /// postal-code, e.g.: "A1A1A1"
  final String location;

  final String kind = "postal-code";

  DomesticDestination({
    @required this.location,
  }) : super(type: "domestic");
}

buildDestination(XmlBuilder builder, Destination destination) {
  builder.element('destination', nest: () {
    builder.element(destination.type, nest: () {
      builder.element(destination.kind, nest: () {
        builder.text(destination.location);
      });
    });
  });
}